package org.apache.bcel.verifier;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.function.Consumer;
import java.util.function.Function;
import org.apache.bcel.Repository;
import org.apache.bcel.classfile.JavaClass;
import org.apache.bcel.classfile.Utility;
import org.apache.bcel.verifier.VerificationResult;
import org.apache.bcel.verifier.VerifierFactory;
import org.apache.bcel.verifier.statics.Pass1Verifier;
import org.apache.bcel.verifier.statics.Pass2Verifier;
import org.apache.bcel.verifier.statics.Pass3aVerifier;
import org.apache.bcel.verifier.structurals.Pass3bVerifier;
import org.apache.commons.lang3.ArrayUtils;

public class Verifier {

   static final Verifier[] EMPTY_ARRAY = new Verifier[0];
   private final String className;
   private Pass1Verifier p1v;
   private Pass2Verifier p2v;
   private final Map p3avs = new HashMap();
   private final Map p3bvs = new HashMap();


   public static void main(String[] args) {
      System.out.println("JustIce by Enver Haase, (C) 2001-2002.\n<http://bcel.sourceforge.net>\n<https://commons.apache.org/bcel>\n");

      for(int index = 0; index < args.length; ++index) {
         try {
            if(args[index].endsWith(".class")) {
               int e = args[index].lastIndexOf(".class");
               if(e != -1) {
                  args[index] = args[index].substring(0, e);
               }
            }

            args[index] = Utility.pathToPackage(args[index]);
            System.out.println("Now verifying: " + args[index] + "\n");
            verifyType(args[index]);
            Repository.clearCache();
            System.gc();
         } catch (ClassNotFoundException var3) {
            var3.printStackTrace();
         }
      }

   }

   static void verifyType(String fullyQualifiedClassName) throws ClassNotFoundException {
      Verifier verifier = VerifierFactory.getVerifier(fullyQualifiedClassName);
      VerificationResult verificationResult = verifier.doPass1();
      System.out.println("Pass 1:\n" + verificationResult);
      verificationResult = verifier.doPass2();
      System.out.println("Pass 2:\n" + verificationResult);
      if(verificationResult == VerificationResult.VR_OK) {
         JavaClass warnings = Repository.lookupClass(fullyQualifiedClassName);

         for(int i = 0; i < warnings.getMethods().length; ++i) {
            verificationResult = verifier.doPass3a(i);
            System.out.println("Pass 3a, method number " + i + " [\'" + warnings.getMethods()[i] + "\']:\n" + verificationResult);
            verificationResult = verifier.doPass3b(i);
            System.out.println("Pass 3b, method number " + i + " [\'" + warnings.getMethods()[i] + "\']:\n" + verificationResult);
         }
      }

      System.out.println("Warnings:");
      String[] var8 = verifier.getMessages();
      if(var8.length == 0) {
         System.out.println("<none>");
      }

      String[] var9 = var8;
      int var5 = var8.length;

      for(int var6 = 0; var6 < var5; ++var6) {
         String warning = var9[var6];
         System.out.println(warning);
      }

      System.out.println("\n");
      verifier.flush();
   }

   Verifier(String fullyQualifiedClassName) {
      this.className = fullyQualifiedClassName;
   }

   public VerificationResult doPass1() {
      if(this.p1v == null) {
         this.p1v = new Pass1Verifier(this);
      }

      return this.p1v.verify();
   }

   public VerificationResult doPass2() {
      if(this.p2v == null) {
         this.p2v = new Pass2Verifier(this);
      }

      return this.p2v.verify();
   }

   public VerificationResult doPass3a(int methodNo) {
      return ((Pass3aVerifier)this.p3avs.computeIfAbsent(Integer.toString(methodNo), (k) -> {
         return new Pass3aVerifier(this, methodNo);
      })).verify();
   }

   public VerificationResult doPass3b(int methodNo) {
      return ((Pass3bVerifier)this.p3bvs.computeIfAbsent(Integer.toString(methodNo), (k) -> {
         return new Pass3bVerifier(this, methodNo);
      })).verify();
   }

   public void flush() {
      this.p1v = null;
      this.p2v = null;
      this.p3avs.clear();
      this.p3bvs.clear();
   }

   public final String getClassName() {
      return this.className;
   }

   public String[] getMessages() throws ClassNotFoundException {
      ArrayList messages = new ArrayList();
      if(this.p1v != null) {
         this.p1v.getMessagesList().forEach((param2) -> {
            messages.add("Pass 1: " + element);
         });
      }

      if(this.p2v != null) {
         this.p2v.getMessagesList().forEach((param2) -> {
            messages.add("Pass 2: " + element);
         });
      }

      Iterator var2 = this.p3avs.values().iterator();

      int meth;
      String[] var5;
      int var6;
      int var7;
      String element;
      while(var2.hasNext()) {
         Pass3aVerifier pv = (Pass3aVerifier)var2.next();
         meth = pv.getMethodNo();
         var5 = pv.getMessages();
         var6 = var5.length;

         for(var7 = 0; var7 < var6; ++var7) {
            element = var5[var7];
            messages.add("Pass 3a, method " + meth + " (\'" + Repository.lookupClass(this.className).getMethods()[meth] + "\'): " + element);
         }
      }

      var2 = this.p3bvs.values().iterator();

      while(var2.hasNext()) {
         Pass3bVerifier var9 = (Pass3bVerifier)var2.next();
         meth = var9.getMethodNo();
         var5 = var9.getMessages();
         var6 = var5.length;

         for(var7 = 0; var7 < var6; ++var7) {
            element = var5[var7];
            messages.add("Pass 3b, method " + meth + " (\'" + Repository.lookupClass(this.className).getMethods()[meth] + "\'): " + element);
         }
      }

      return (String[])messages.toArray(ArrayUtils.EMPTY_STRING_ARRAY);
   }

}
